@@ -23,6 +23,9 @@ from .transformers import (vit_imagenet, vit_ti16, bc_vit_ti16,  bc_vit_ti16_ima
                            deit_b16)
 from .portrait128 import akida_unet_portrait128, akida_unet_portrait128_pretrained
 from .urbansound import vit_urbansound_pretrained
+from .tenn_spatiotemporal import (tenn_spatiotemporal_dvs128, tenn_spatiotemporal_dvs128_pretrained,
+                                  tenn_spatiotemporal_eye, tenn_spatiotemporal_eye_pretrained,
+                                  tenn_spatiotemporal_jester, tenn_spatiotemporal_jester_pretrained)
 
 from .utils import fetch_file
 from .model_io import load_model
