@@ -25,6 +25,8 @@ specified location.
 import argparse
 import os
 import sys
+import onnx
+import numpy as np
 
 from .utk_face.model_vgg import vgg_utk_face
 from .kws.model_ds_cnn import ds_cnn_kws
@@ -40,10 +42,11 @@ from .transformers.model_vit import vit_ti16, bc_vit_ti16
 from .transformers.model_deit import deit_ti16, bc_deit_ti16
 from .portrait128.model_akida_unet import akida_unet_portrait128
 from .centernet.model_centernet import centernet_base
-from .macs import display_macs
+from .sparsity import compute_sparsity
 from .model_io import load_weights, save_weights, load_model
 from .unfuse_sepconv_layers import unfuse_sepconv2d
-import onnx
+from .tenn_spatiotemporal import (tenn_spatiotemporal_dvs128, tenn_spatiotemporal_eye,
+                                  tenn_spatiotemporal_jester)
 
 
 def save_model_weights(model_path, weights_path):
@@ -240,6 +243,19 @@ def main():
                            choices=[224, 384],
                            help="The square input image size")
 
+    dvst_parser = csp.add_parser("tenn_spatiotemporal_dvs128",
+                                 help="A TENN spatiotemporal model for DVS128")
+    add_common_args(dvst_parser, default_classes=10)
+
+    eye_p = csp.add_parser("tenn_spatiotemporal_eye",
+                           help="A TENN spatiotemporal model for AIS2024 event-based eyetracking")
+    eye_p.add_argument("-reg", "--reg_factor", type=float, default=1e-8,
+                       help="The regularization factor, default %(default)s.")
+
+    jester_parser = csp.add_parser("tenn_spatiotemporal_jester",
+                                   help="A TENN spatiotemporal model for jester video.")
+    add_common_args(jester_parser, default_classes=27)
+
     vit_parser = csp.add_parser("vit_ti16", help="A ViT model")
     add_transformers_args(vit_parser, add_customs=True)
     vit_parser.add_argument("-i", "--image_size", type=int, default=224,
@@ -266,9 +282,15 @@ def main():
 
     csp.add_parser("vit_urbansound", help="An audio ViT, brainchip customized model")
 
-    macs_parser = sp.add_parser("macs", help="Get MACS for a Keras model")
-    macs_parser.add_argument("-m", "--model", type=str)
-    macs_parser.add_argument("-v", "--verbose", action="store_true")
+    sparsity_parser = sp.add_parser("sparsity", help="Compute the sparsity of the model")
+    sparsity_parser.add_argument("-m", "--model", type=str,
+                                 help="Model which sparsity to compute.")
+    sparsity_parser.add_argument("-l", "--layer_names", type=str, default=None,
+                                 help="Comma-separated list of layers")
+    sparsity_parser.add_argument("-ns", "--n_samples", type=int, default=10,
+                                 help="The number of the samples used to compute the sparsity")
+    sparsity_parser.add_argument("-sa", "--samples", type=str, default=None,
+                                 help="The path of the samples file used to compute the sparsity")
 
     unfuse_parser = sp.add_parser(
         "unfuse", help="Unfuse SeparableConv2D layers of a Keras model")
@@ -342,6 +364,12 @@ def main():
             model = akida_unet_portrait128()
             if args.base_weights is not None:
                 model.load_weights(args.base_weights, by_name=True)
+        elif args.model == "tenn_spatiotemporal_dvs128":
+            model = tenn_spatiotemporal_dvs128()
+        elif args.model == "tenn_spatiotemporal_eye":
+            model = tenn_spatiotemporal_eye(reg_factor=args.reg_factor)
+        elif args.model == "tenn_spatiotemporal_jester":
+            model = tenn_spatiotemporal_jester()
         elif args.model == "vit_urbansound":
             # Create a base ViT model with 10 classes and 1 transformer block
             model = bc_vit_ti16(input_shape=(224, 224, 1), classes=10, num_blocks=1)
@@ -385,8 +413,12 @@ def main():
         model.save(model_path, include_optimizer=False)
         print(f"Model saved as {model_path}")
 
-    elif args.action == "macs":
-        display_macs(args.model, args.verbose)
+    elif args.action == "sparsity":
+        if args.layer_names:
+            args.layer_names = args.layer_names.split(',')
+        model = load_model(args.model, compile_model=False)
+        samples = np.load(args.samples)['data'] if args.samples else None
+        compute_sparsity(model, args.layer_names, None, samples, args.n_samples, verbose=True)
     elif args.action == "save_weights":
         save_model_weights(
             model_path=args.model,
