@@ -14,13 +14,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ******************************************************************************
-"""
- DVS DataGenerator class.
+""" DVS DataGenerator class.
 
- This script is used in dvs_train.py to generate images from DVS data from NPY
-    files. It's mandatory because Tensorflow doesn't handle events data.
+This script is used in dvs_train.py to generate images from DVS data from NPY files. It's mandatory
+because Tensorflow doesn't handle events data.
 """
 
+__all__ = ["DVS_DataGenerator"]
+
 import random
 import numpy as np
 
