@@ -32,7 +32,8 @@ except ImportError:
 import keras
 from keras.callbacks import LearningRateScheduler
 from keras import Sequential
-from keras.optimizers import SGD, Adam
+from keras.optimizers import SGD
+from keras.optimizers.legacy import Adam
 from keras.layers import Input
 from keras.models import clone_model
 
