@@ -24,7 +24,10 @@ from keras.layers import (BatchNormalization, ReLU, Conv2D, DepthwiseConv2D, Sep
                           Conv2DTranspose, Reshape, Conv3D)
 from keras.activations import swish, gelu
 from keras.initializers import TruncatedNormal
+from keras.utils import register_keras_serializable
 import tensorflow as tf
+import numpy as np
+from scipy.special import jacobi
 
 from quantizeml.layers import (Attention, LayerMadNormalization, DepthwiseConv2DTranspose,
                                ExtractToken)
@@ -590,11 +593,105 @@ def vit_extract_feature_map(y, feat_shape, num_non_patch=1):
     return y
 
 
+def get_ortho_polynomials(length, degrees, alpha, beta):
+    """ Generate the set of Jacobi orthogonal polynomials with shape (degrees + 1, length)
+
+    Args:
+        length (int): The length of the discretized temporal kernel,
+            assuming the range [0, 1] for the polynomials.
+        degrees (int): The maximum polynomial degree. Note that degrees + 1 polynomials
+            will be generated (counting the constant).
+        alpha (float): The alpha Jacobi parameter.
+        beta (float): The beta Jacobi parameter.
+
+    Returns:
+        np.ndarray: shaped (degrees + 1, length)
+    """
+    coeffs = np.vstack([np.pad(np.flip(jacobi(degree, alpha, beta).coeffs), (0, degrees - degree))
+                        for degree in range(degrees + 1)]).astype(np.float32)
+    steps = np.linspace(0, 1, length + 1)
+    X = np.stack([steps ** (i + 1) / (i + 1) for i in range(degrees + 1)])
+    polynomials_integrated = coeffs @ X
+    transform = np.diff(polynomials_integrated, 1, -1) * length
+    return transform
+
+
+@register_keras_serializable()
+class PleiadesLayer(Conv3D):
+    """A 3D convolutional layer utilizing orthogonal polynomials for kernel transformation.
+
+    Inherits from `Conv3D` and modifies its kernel transformation before applying convolution.
+
+    Args:
+        filters (int): Number of output filters.
+        kernel_size (tuple): Size of the convolution kernel.
+        degrees (int): Degree of the orthogonal polynomials.
+        alpha (float): Alpha parameter for the orthogonal polynomials.
+        beta (float): Beta parameter for the orthogonal polynomials.
+        **kwargs: Additional arguments passed to `Conv3D`.
+    """
+    def __init__(self, filters, kernel_size, degrees, alpha, beta, **kwargs):
+        super().__init__(filters, kernel_size, **kwargs)
+
+        self.degrees = degrees
+        self.alpha = alpha
+        self.beta = beta
+
+    def build(self, input_shape):
+        with tf.name_scope(self.name + '/'):
+            super().build(input_shape)
+            # Generate the transformation matrix
+            transform = get_ortho_polynomials(self.kernel_size[0], self.degrees, self.alpha,
+                                              self.beta)
+            transform = tf.convert_to_tensor(transform, dtype=tf.float32)
+            # Normalize transform based on input dimensions
+            scale = tf.sqrt(tf.cast(input_shape[-1],
+                                    tf.float32)) * tf.sqrt(tf.cast(self.kernel_size[0],
+                                                                   tf.float32))
+            self.transform = transform / scale
+            new_kernel_shape = (self.kernel.shape[-2], self.filters, self.kernel_size[1],
+                                self.kernel_size[2], self.degrees + 1)
+            self.kernel = self.add_weight(shape=new_kernel_shape,
+                                          initializer=self.kernel_initializer,
+                                          trainable=True, name="kernel")
+
+    def call(self, inputs):
+        # Apply polynomial transformation to kernel
+        kernel = tf.tensordot(self.kernel, self.transform, axes=[[4], [0]])
+        kernel = tf.transpose(kernel, perm=[4, 2, 3, 0, 1])
+        # Perform convolution with transformed kernel
+        if self.groups > 1:
+            # The '_jit_compiled_convolution_op' is a specialized operation that efficiently
+            # handles grouped convolutions, addressing limitations in some TensorFlow versions
+            # where grouped convolutions are not directly supported.
+            conv_output = self._jit_compiled_convolution_op(inputs, kernel)
+        else:
+            conv_output = self.convolution_op(inputs, kernel)
+        if self.use_bias:
+            conv_output = tf.nn.bias_add(conv_output, self.bias)
+        return conv_output
+
+    def get_config(self):
+        config = super().get_config()
+        config.update({
+            'degrees': self.degrees,
+            'alpha': self.alpha,
+            'beta': self.beta
+        })
+        return config
+
+
 def conv3d_block(inputs,
                  filters,
                  kernel_size,
                  add_batchnorm=False,
                  relu_activation='ReLU3.75',
+                 reg_factor=None,
+                 normalize_reg=False,
+                 use_pleiades=False,
+                 degrees=None,
+                 alpha=None,
+                 beta=None,
                  **kwargs):
     """Adds a Conv3D layer with optional layers: batch normalization and activation.
 
@@ -606,6 +703,19 @@ def conv3d_block(inputs,
         relu_activation (str, optional): the ReLU activation to add to the layer in the form 'ReLUx'
             where 'x' is the max_value to use. Set to False to disable activation. Defaults to
             'ReLU3.75'.
+        reg_factor (float, optional): the L1-regularization factor of the ActivityRegularization
+            layers that are added after the ReLU layers if reg_factor is not None.
+            Defaults to None.
+        normalize_reg (bool, optional): if True, normalize the L1-regularization factor.
+            Defaults to False.
+        use_pleiades (bool, optional): if True, the first conv3d of the TemporalBlock
+            is a PleiadesLayer. Defaults to False.
+        degrees (int, optional): The maximum polynomial degree of the Jacobi matrix. It needs to be
+            set if use_pleiades is True. Defaults to None.
+        alpha (int, optional): The alpha Jacobi parameter. It needs to be set if
+            use_pleiades is True. Defaults to None.
+        beta (int, optional): The beta Jacobi parameter. It needs to be set if use_pleiades is True.
+            Defaults to None.
         **kwargs: arguments passed to the keras.Conv3D layer, such as strides, use_bias, etc.
 
     Returns:
@@ -620,7 +730,11 @@ def conv3d_block(inputs,
             padding=((kernel_size[0] - 1, 0), (0, 0), (0, 0)))
         inputs = padding_layer(inputs)
         kwargs['padding'] = 'valid'
-    conv_layer = Conv3D(filters, kernel_size, **kwargs)
+    if use_pleiades:
+        assert degrees and alpha and beta
+        conv_layer = PleiadesLayer(filters, kernel_size, degrees, alpha, beta, **kwargs)
+    else:
+        conv_layer = Conv3D(filters, kernel_size, **kwargs)
     x = conv_layer(inputs)
 
     if add_batchnorm:
@@ -628,12 +742,16 @@ def conv3d_block(inputs,
 
     if relu_activation:
         x = act_to_layer(relu_activation, name=conv_layer.name + '/relu')(x)
-
+        if reg_factor:
+            if normalize_reg:
+                reg_factor /= np.prod(x.shape[1:])
+            x = tf.keras.layers.ActivityRegularization(l1=reg_factor)(x)
     return x
 
 
-def spatiotemporal_block(inputs, in_channels, med_channels, out_channels, t_kernel_size, t_stride,
-                         t_depthwise, s_depthwise, index):
+def spatiotemporal_block(inputs, in_channels, med_channels, out_channels, t_kernel_size,
+                         t_depthwise, s_depthwise, index, reg_factor=None, normalize_reg=False,
+                         use_pleiades=False, degrees=None, alpha=None, beta=None):
     """ Add a spatiotemporal block to the inputs.
 
     The spatio-temporal block consists of a temporal convolution (potentially separable) followed by
@@ -648,10 +766,22 @@ def spatiotemporal_block(inputs, in_channels, med_channels, out_channels, t_kern
         med_channels (int): middle channels (channels after the temporal conv layer)
         out_channels (int): output channels (channels after the spatial conv layer)
         t_kernel_size (int): the temporal kernel size
-        t_stride (int): the temporal kernel stride
         t_depthwise (bool): whether the temporal layer is dw_separable
         s_depthwise (bool): whether the spatial layer is dw_separable
         index (int): index of the block
+        reg_factor (float, optional): the L1-regularization factor of the ActivityRegularization
+            layers that are added after the ReLU layers if reg_factor is not None.
+            Defaults to None.
+        normalize_reg (bool, optional): if True, normalize the L1-regularization factor.
+            Defaults to False.
+        use_pleiades (bool, optional): if True, the first conv3d of the TemporalBlock
+            is a PleiadesLayer. Defaults to False.
+        degrees (int, optional): The maximum polynomial degree of the Jacobi matrix. It needs to be
+            set if use_pleiades is True. Defaults to None.
+        alpha (int, optional): The alpha Jacobi parameter. It needs to be set if
+            use_pleiades is True. Defaults to None.
+        beta (int, optional): The beta Jacobi parameter. It needs to be set if use_pleiades is True.
+            Defaults to None.
 
     Returns:
         tf.Tensor: output tensor of the spatiotemporal block.
@@ -662,10 +792,16 @@ def spatiotemporal_block(inputs, in_channels, med_channels, out_channels, t_kern
                          (t_kernel_size, 1, 1),
                          add_batchnorm=True,
                          relu_activation='ReLU',
-                         strides=(t_stride, 1, 1),
+                         strides=(1, 1, 1),
                          padding='same',
                          use_bias=True,
-                         name=f'convt_full_{index}')
+                         name=f'convt_full_{index}',
+                         reg_factor=reg_factor,
+                         normalize_reg=normalize_reg,
+                         use_pleiades=use_pleiades,
+                         degrees=degrees,
+                         alpha=alpha,
+                         beta=beta)
     else:
         # This is a DepthwiseConv3D (groups=filters)
         x = conv3d_block(inputs,
@@ -673,17 +809,25 @@ def spatiotemporal_block(inputs, in_channels, med_channels, out_channels, t_kern
                          (t_kernel_size, 1, 1),
                          add_batchnorm=True,
                          relu_activation='ReLU',
-                         strides=(t_stride, 1, 1),
+                         strides=(1, 1, 1),
                          padding='same',
                          groups=in_channels,
                          use_bias=False,
-                         name=f'convt_dw_{index}')
+                         name=f'convt_dw_{index}',
+                         reg_factor=reg_factor,
+                         normalize_reg=normalize_reg,
+                         use_pleiades=use_pleiades,
+                         degrees=degrees,
+                         alpha=alpha,
+                         beta=beta)
         x = conv3d_block(x,
                          med_channels,
                          (1, 1, 1),
                          add_batchnorm=True,
                          relu_activation='ReLU',
-                         name=f'convt_pw_{index}')
+                         name=f'convt_pw_{index}',
+                         reg_factor=reg_factor,
+                         normalize_reg=normalize_reg)
 
     if not s_depthwise:
         x = conv3d_block(x,
@@ -694,7 +838,9 @@ def spatiotemporal_block(inputs, in_channels, med_channels, out_channels, t_kern
                          strides=(1, 2, 2),
                          padding='same',
                          use_bias=True,
-                         name=f'convs_full_{index}')
+                         name=f'convs_full_{index}',
+                         reg_factor=reg_factor,
+                         normalize_reg=normalize_reg)
     else:
         # This is a DepthwiseConv3D (groups=filters)
         x = conv3d_block(x,
@@ -706,11 +852,15 @@ def spatiotemporal_block(inputs, in_channels, med_channels, out_channels, t_kern
                          padding='same',
                          groups=med_channels,
                          use_bias=False,
-                         name=f'convs_dw_{index}')
+                         name=f'convs_dw_{index}',
+                         reg_factor=reg_factor,
+                         normalize_reg=normalize_reg)
         x = conv3d_block(x,
                          out_channels,
                          (1, 1, 1),
                          add_batchnorm=True,
                          relu_activation='ReLU',
-                         name=f'convs_pw_{index}')
+                         name=f'convs_pw_{index}',
+                         reg_factor=reg_factor,
+                         normalize_reg=normalize_reg)
     return x
