@@ -22,12 +22,14 @@ import os
 from keras.datasets import mnist
 from keras.losses import SparseCategoricalCrossentropy
 
-from cnn2snn import load_quantized_model, convert
+from cnn2snn import convert
 
 import numpy as np
 
 from ..training import (get_training_parser, compile_model, evaluate_model, print_history_stats,
                         save_model, RestoreBest)
+from ..extract import extract_samples
+from ..model_io import load_model
 
 
 def get_data():
@@ -69,11 +71,11 @@ def train_model(model, x_train, y_train, x_test, y_test, batch_size, epochs):
 def main():
     """ Entry point for script and CLI usage.
     """
-    parsers = get_training_parser(batch_size=128, tune=True, global_batch_size=False)
+    parsers = get_training_parser(batch_size=128, tune=True, global_batch_size=False, extract=True)
     args = parsers[0].parse_args()
 
     # Load the source model
-    model = load_quantized_model(args.model)
+    model = load_model(args.model)
 
     # Compile model
     lr = 1e-4
@@ -99,6 +101,9 @@ def main():
             print(f"Accuracy: {accuracy}")
         else:
             evaluate_model(model, x_test, y=y_test)
+    elif args.action == 'extract':
+        # Extract samples from dataset
+        extract_samples(args.savefile, x_train, args.batch_size)
 
 
 if __name__ == "__main__":
