@@ -20,16 +20,18 @@ ModelNet40 model training script.
 
 import os
 import argparse
-import keras
 import numpy as np
 
 from keras.callbacks import LearningRateScheduler
 
-from cnn2snn import load_quantized_model, convert
+from cnn2snn import convert
 
 from .preprocessing import get_modelnet, get_modelnet_from_file
 
-from ..training import get_training_parser, compile_model, print_history_stats
+from ..training import (get_training_parser, compile_model, print_history_stats, evaluate_model,
+                        evaluate_akida_model, save_model)
+from ..extract import extract_samples
+from ..model_io import load_model
 
 
 def get_data(batch_size, selected_points, knn_points):
@@ -86,33 +88,6 @@ def train_model(model, train_dataset, test_dataset, epochs):
     print_history_stats(history)
 
 
-def evaluate_model(model, test_dataset):
-    """ Evaluates model performances.
-
-    Args:
-        model (keras.Model or akida.Model): the model to evaluate.
-        test_dataset (tf.dataset): evaluation dataset.
-    """
-    labels = np.concatenate([labels for x, labels in test_dataset], axis=0)
-
-    if isinstance(model, keras.Model):
-        preds = np.squeeze(np.argmax(model.predict(test_dataset), axis=1))
-    else:
-        pots_akida = np.empty(shape=[0, model.output_shape[-1]],
-                              dtype=np.float32)
-
-        for batch, _ in test_dataset:
-            # Rescale inputs to Akida
-            pots = (batch.numpy() * 127 + 127).astype('uint8')
-            pots_batch = model.predict(pots)
-            pots_akida = np.concatenate((pots_akida, pots_batch.squeeze()))
-
-        preds = np.squeeze(np.argmax(pots_akida, 1))
-
-    acc = np.sum(preds == labels) / labels.shape[0]
-    print(f"Accuracy: {acc}")
-
-
 def main():
     """ Entry point for script and CLI usage.
     """
@@ -121,21 +96,21 @@ def main():
         "-p",
         "--selected_points",
         type=int,
-        default=128,
+        default=64,
         help="The number of points to process per sample. \
-                        (default 128)")
+                        (defaults to %(default)s)")
     global_parser.add_argument("-n",
                                "--knn_points",
                                type=int,
-                               default=64,
+                               default=32,
                                help="The number of points to include in each \
-                        localised group. (default 64)")
+                        localised group. (defaults to %(default)s)")
 
-    parser = get_training_parser(batch_size=16, global_parser=global_parser)[0]
+    parser = get_training_parser(batch_size=16, global_parser=global_parser, extract=True)[0]
     args = parser.parse_args()
 
     # Load the source model
-    model = load_quantized_model(args.model)
+    model = load_model(args.model)
 
     # Compile model
     compile_model(model, loss="sparse_categorical_crossentropy")
@@ -151,17 +126,19 @@ def main():
     # Train model
     if args.action == "train":
         train_model(model, train_dataset, test_dataset, args.epochs)
-
-        # Save model in Keras format (h5)
-        if args.savemodel is not None:
-            model.save(args.savemodel, include_optimizer=False)
-            print(f"Trained model saved as {args.savemodel}")
-
+        save_model(model, args.model, args.savemodel, args.action)
     elif args.action == "eval":
         # Evaluate model accuracy
         if args.akida:
-            model = convert(model, input_scaling=(127, 127))
-        evaluate_model(model, test_dataset)
+            model = convert(model)
+            preds, labels = evaluate_akida_model(model, test_dataset, 'softmax')
+            accuracy = (np.argmax(preds, 1) == labels).mean()
+            print(f"Akida accuracy: {accuracy}")
+        else:
+            evaluate_model(model, test_dataset)
+    elif args.action == 'extract':
+        # Extract samples from dataset
+        extract_samples(args.savefile, train_dataset, args.batch_size)
 
 
 if __name__ == "__main__":
