@@ -17,10 +17,12 @@
 Utils for PointNet.
 """
 
+__all__ = ["get_reshape_factor", "pointnet_preproc"]
+
 import tensorflow as tf
 
 
-def shift_to_7bit(values, val_min, val_max):
+def _shift_to_7bit(values, val_min, val_max):
     """ Normalize data to 7 bits
     Args:
         values (tf.Tensor): input data to normalize.
@@ -93,7 +95,7 @@ def pointnet_preproc(points, label, selected_points, knn_points):
     # Also, the preprocessing function performs a substraction between point
     # coordinates, so some values are guaranteed to be negative at that point
     # in the pipeline.
-    points = shift_to_7bit(points, -1.0, 1.0)
+    points = _shift_to_7bit(points, -1.0, 1.0)
 
     # Random point selector
     # Simplification: selects same indices within a batch, and doesn't do
@@ -137,7 +139,7 @@ def pointnet_preproc(points, label, selected_points, knn_points):
         subtraction_helper = tf.concat(
             [subtraction_helper, subtraction_padding], -1)
 
-    outpoints = (outpoints - subtraction_helper) / 127.0
+    outpoints = outpoints - subtraction_helper
 
     reshape_factor = get_reshape_factor(knn_points)
 
@@ -148,6 +150,11 @@ def pointnet_preproc(points, label, selected_points, knn_points):
         -1, num_features, selected_points * reshape_factor,
         int(knn_points / reshape_factor)
     ])
-    # put dimensions back to original order
-    points = tf.transpose(x, perm=[0, 2, 3, 1])
+    # put dimensions to:
+    # (knn_points // reshape_factor) x (selected_points * reshape_factor) x features
+    points = tf.transpose(x, perm=[0, 3, 2, 1])
+
+    # Go back to [0, 255] uint8 range for homogeneity with Akida. The model will have a rescaling
+    # layer to get back to [-1, 1].
+    points = points + 127
     return points, label
