@@ -24,6 +24,7 @@ import argparse
 import tensorflow as tf
 import numpy as np
 from keras.metrics import BinaryIoU
+from keras.losses import BinaryCrossentropy
 
 from cnn2snn import convert
 
@@ -43,7 +44,7 @@ def _train_eval_split(x, y, validation_split=0.2, random_state=None):
         y (numpy.ndarray): The corresponding labels.
         validation_split (float, optional): Fraction of the original dataset
             used as the validation set. Should be in the range [0, 1]. Defaults to 0.2
-        random_state (int, optional): Seed for reproducibility. Default to None.
+        random_state (int, optional): Seed for reproducibility. Defaults to None.
 
     Returns:
         (numpy.ndarray, numpy.ndarray, numpy.ndarray, numpy.ndarray): Train and eval sets.
@@ -82,19 +83,17 @@ def get_data(path, batch_size):
                                                        y,
                                                        validation_split=0.2,
                                                        random_state=seed)
+    data_augmentation = tf.keras.Sequential([
+        tf.keras.layers.RandomFlip("horizontal", seed=seed),
+        tf.keras.layers.RandomTranslation(height_factor=0.1, width_factor=0.1, seed=seed),
+        tf.keras.layers.RandomZoom(0.2, seed=seed),
+    ])
 
     def data_generator(images, masks):
         for image, mask in zip(images, masks):
             yield image, mask
 
     def apply_data_augmentation(image, mask):
-        data_augmentation = tf.keras.Sequential(
-            [
-                tf.keras.layers.RandomFlip("horizontal", seed=seed),
-                tf.keras.layers.RandomTranslation(height_factor=0.1, width_factor=0.1, seed=seed),
-                tf.keras.layers.RandomZoom(0.2, seed=seed),
-            ]
-        )
         # Concatenate image and mask along the last axis
         combined = tf.concat([image, mask], axis=-1)
         # Apply the same augmentation to both image and mask
@@ -186,14 +185,12 @@ def evaluate_akida_model(model, val_ds, val_steps):
         else:
             labels = np.concatenate((labels, label_batch))
             pots = np.concatenate((pots, pots_batch))
-    preds = tf.keras.activations.sigmoid(pots)
-
-    m_binary_iou = tf.keras.metrics.BinaryIoU(target_class_ids=[0, 1], threshold=0.5)
-    m_binary_iou.update_state(labels, preds)
+    m_binary_iou = tf.keras.metrics.BinaryIoU(target_class_ids=[0, 1], threshold=0)
+    m_binary_iou.update_state(labels, pots)
     binary_iou = m_binary_iou.result().numpy()
 
     m_accuracy = tf.keras.metrics.Accuracy()
-    m_accuracy.update_state(labels, preds > 0.5)
+    m_accuracy.update_state(labels, pots > 0)
     accuracy = m_accuracy.result().numpy()
     print(f"Akida BinaryIoU/pixel accuracy: {binary_iou:.4f}/{100*accuracy:.2f}%")
 
@@ -221,7 +218,8 @@ def main():
     if args.action == "tune":
         learning_rate /= 10
 
-    compile_model(model, learning_rate=learning_rate, loss='binary_crossentropy',
+    compile_model(model, learning_rate=learning_rate,
+                  loss=BinaryCrossentropy(from_logits=True),
                   metrics=[BinaryIoU(), 'accuracy'])
 
     # Load data
