@@ -24,7 +24,7 @@ import argparse
 import numpy as np
 
 from keras.preprocessing.image import DirectoryIterator
-from keras.optimizers import Adam
+from keras.optimizers.legacy import Adam
 from keras.layers import Activation
 from keras.callbacks import ModelCheckpoint
 
