@@ -18,17 +18,20 @@
 ViT model definition.
 Inspired from https://github.com/faustomorales/vit-keras/blob/master/vit_keras/vit.py.
 """
+
+__all__ = ["deit_imagenet", "deit_ti16", "bc_deit_ti16", "bc_deit_dist_ti16_imagenet_pretrained",
+           "deit_s16", "deit_b16"]
+
 import keras
 
 from quantizeml.layers import AddPositionEmbs, ClassToken, Add, ExtractToken
-from quantizeml.models import load_model
+from quantizeml import load_model
 
 from .model_vit import CONFIG_TI, CONFIG_S, CONFIG_B
 from ..imagenet.imagenet_utils import IMAGENET_MEAN, IMAGENET_STD
 from ..layer_blocks import norm_to_layer, transformer_block
 from ..utils import fetch_file
-
-BASE_WEIGHT_PATH = 'http://data.brainchip.com/models/AkidaV2/deit/'
+from ..model_io import get_model_path
 
 
 def deit_imagenet(input_shape,
@@ -46,7 +49,17 @@ def deit_imagenet(input_shape,
                   last_norm='LN',
                   softmax='softmax',
                   act="GeLU"):
-    """Build a DeiT model.
+    """Instantiates the DeiT architecture.
+
+    The Data-efficient image Transformers (DeiT) is a model for image classification,
+    requiring far less data and far less computing resources compared to the original
+    ViT model. It relies on a teacher-student strategy specific to transformers
+    (distillation token).
+
+    Please refer to https://arxiv.org/abs/2012.12877 for further details.
+
+    Note: input preprocessing is included as part of the model (as a Rescaling layer). This model
+    expects inputs to be float tensors of pixels with values in the [0, 255] range.
 
     Args:
         input_shape (tuple): image shape tuple
@@ -141,7 +154,8 @@ def deit_ti16(input_shape=(224, 224, 3),
               softmax='softmax',
               act='GeLU',
               include_top=True):
-    """Build DeiT-Tiny.
+    """Instantiates the DeiT-Tiny 16 architecture; that is a DeiT architecture with 3 attention heads,
+    12 blocks and a patch size of 16.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
@@ -179,7 +193,12 @@ def deit_ti16(input_shape=(224, 224, 3),
 
 def bc_deit_ti16(input_shape=(224, 224, 3), classes=1000, distilled=False, include_top=True,
                  num_blocks=12):
-    """Build DeiT-Tiny, changing all LN by LMN, using softmax2 and ReLU8.
+    """Instantiates the DeiT-Tiny 16 architecture adapted for implementation on hardware,
+    that is:
+
+        - LayerNormalization replaced by LayerMadNormalization,
+        - GeLU replaced by ReLU8 activations,
+        - Softmax replaced by shiftmax.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
@@ -209,16 +228,33 @@ def bc_deit_ti16(input_shape=(224, 224, 3), classes=1000, distilled=False, inclu
     )
 
 
-def bc_deit_dist_ti16_imagenet_pretrained():
-    """ Helper method to retrieve a `bc_deit_dist_ti16` model that was trained on ImageNet dataset.
+def bc_deit_dist_ti16_imagenet_pretrained(quantized=True):
+    """ Helper method to retrieve a DeiT-Tiny 16 model adapted for implementation on hardware,
+    that is:
+
+        - LayerNormalization replaced by LayerMadNormalization,
+        - GeLU replaced by ReLU8 activations,
+        - Softmax replaced by shiftmax,
+
+    and that was trained on ImageNet dataset.
+
+    Args:
+        quantized (bool, optional): a boolean indicating whether the model should be loaded
+            quantized or not. Defaults to True.
 
     Returns:
-        keras.Model, dict: a Keras Model instance and the quantization configuration as a JSON
-        formatted dict.
+        keras.Model: a Keras Model instance
     """
-    model_name = 'bc_deit_dist_ti16_224_i8_w4_a4.h5'
-    file_hash = '0975fa986c8d026929568ae47022b153a39d9bb6327cb2aa8b0a38b3c03f6025'
-    model_path = fetch_file(BASE_WEIGHT_PATH + model_name,
+    if quantized:
+        model_name_v2 = 'bc_deit_dist_ti16_224_i8_w8_a8.h5'
+        file_hash_v2 = '911cd14d3223831789c11234a9c99a6dd007ce9baec8a8d5b5f208644befaf54'
+    else:
+        model_name_v2 = 'bc_deit_dist_ti16_224.h5'
+        file_hash_v2 = '40618ba14d894fc308ef149c1eb17ff601ad54a8bc0ad329dc6f7dba5ff86260'
+
+    model_path, model_name, file_hash = get_model_path("deit", model_name_v2=model_name_v2,
+                                                       file_hash_v2=file_hash_v2)
+    model_path = fetch_file(model_path,
                             fname=model_name,
                             file_hash=file_hash,
                             cache_subdir='models')
@@ -226,7 +262,8 @@ def bc_deit_dist_ti16_imagenet_pretrained():
 
 
 def deit_s16(input_shape=(224, 224, 3), classes=1000, distilled=False, include_top=True):
-    """Build DeiT-Small.
+    """Instantiates the DeiT-Small 16 architecture; that is a ViT architecture with 6 attention heads,
+    12 blocks and a patch size of 16.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
@@ -249,7 +286,8 @@ def deit_s16(input_shape=(224, 224, 3), classes=1000, distilled=False, include_t
 
 
 def deit_b16(input_shape=(224, 224, 3), classes=1000, distilled=False, include_top=True):
-    """Build DeiT-B16.
+    """Instantiates the DeiT-B16 architecture; that is a ViT architecture with 12 attention heads,
+    12 blocks and a patch size of 16.
 
     Args:
         input_shape (tuple, optional): input shape. Defaults to (224, 224, 3).
